@@ -188,9 +188,9 @@ let test_pts () =
 
 let test_execinfo () =
   require "backtrace";
-  (* backtrace may not work out of the box on all archs, see PR#5334 *)
+  (* Disabled - may not work out of the box on all archs or even segfault *)
 (*   assert_bool "backtrace" ([||] <> backtrace ()) *)
-  let (_:string array) = backtrace () in
+(*   let (_:string array) = backtrace () in *)
   ()
 
 let test_statvfs () =
@@ -301,21 +301,18 @@ let test_endian_string () =
   assert_equal (L.get_int16  src  4) (-0x2302);
   assert_equal (L.get_int32  src  6) (0x98BADCFEl);
   assert_equal (L.get_int64  src 10) (0x1032547698BADCFEL);
-  if Sys.word_size = 64 then
-  begin
-    assert_equal (B.get_uint31  src  6) 0xFEDCBA98;
-    assert_equal (B.get_int31   src  6) 0x7FFFFFFFFEDCBA98;
-    assert_equal (B.get_int31   src  6) (-0x1234568);
-    assert_equal (B.get_uint63  src 10) 0x7EDCBA9876543210;
-    assert_equal (B.get_int63   src 10) 0x7EDCBA9876543210;
-    assert_equal (B.get_int63   src 10) (-0x123456789ABCDF0);
-    assert_equal (L.get_uint31  src  6) 0x98BADCFE;
-    assert_equal (L.get_int31   src  6) 0x7FFFFFFF98BADCFE;
-    assert_equal (L.get_int31   src  6) (-0x67452302);
-    assert_equal (L.get_uint63  src 10) 0x1032547698BADCFE;
-    assert_equal (L.get_int63   src 10) 0x1032547698BADCFE;
-    assert_equal (L.get_int63   src 10) (-0x6FCDAB8967452302);
-  end;
+  assert_equal (B.get_uint31 src  6) (Int64.to_int 0xFEDCBA98L);
+  assert_equal (B.get_int31  src  6) (Int64.to_int 0x7FFFFFFFFEDCBA98L);
+  assert_equal (B.get_int31  src  6) (-0x1234568);
+  assert_equal (B.get_uint63 src 10) (Int64.to_int 0x7EDCBA9876543210L);
+  assert_equal (B.get_int63  src 10) (Int64.to_int 0x7EDCBA9876543210L);
+  assert_equal (B.get_int63  src 10) (Int64.to_int (-0x123456789ABCDF0L));
+  assert_equal (L.get_uint31 src  6) (Int64.to_int 0x98BADCFEL);
+  assert_equal (L.get_int31  src  6) (Int64.to_int 0x7FFFFFFF98BADCFEL);
+  assert_equal (L.get_int31  src  6) (-0x67452302);
+  assert_equal (L.get_uint63 src 10) (Int64.to_int 0x1032547698BADCFEL);
+  assert_equal (L.get_int63  src 10) (Int64.to_int 0x1032547698BADCFEL);
+  assert_equal (L.get_int63  src 10) (Int64.to_int (-0x6FCDAB8967452302L));
   let b = "                  " in
   B.set_uint8  b  0 0xFF;
   B.set_int8   b  1 (-0x01);
@@ -510,6 +507,21 @@ let test_memalign () =
   ignore (memalign 2048 16384);
   ignore (memalign 4096 65536)
 
+let test_sockopt () =
+  require "setsockopt_int";
+  require "getsockopt_int";
+  let fd = Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
+  Unix.setsockopt fd Unix.SO_KEEPALIVE true;
+  setsockopt_int fd TCP_KEEPCNT 5;
+  setsockopt_int fd TCP_KEEPIDLE 30;
+  setsockopt_int fd TCP_KEEPINTVL 10;
+  let test msg opt v =
+    assert_equal ~printer:string_of_int ~msg v (getsockopt_int fd opt)
+  in
+  test "TCP_KEEPCNT" TCP_KEEPCNT 5;
+  test "TCP_KEEPIDLE" TCP_KEEPIDLE 30;
+  test "TCP_KEEPINTVL" TCP_KEEPINTVL 10
+
 let () =
   let wrap test =
     with_unix_error (fun () -> test (); Gc.compact ())
@@ -541,6 +553,7 @@ let () =
     "mkstemp" >:: test_mkstemp;
     "mkostemp" >:: test_mkostemp;
     "memalign" >:: test_memalign;
+    "sockopt" >:: test_sockopt;
 ]) in
   ignore (run_test_tt_main (test_decorate wrap tests))
 
